#![allow(clippy::too_many_arguments)]

use std::{collections::HashMap, time::Duration};

use anyhow::Result;
use tokio::{
    process::Command,
    sync::mpsc::{self, Receiver, Sender},
};
use zbus::Connection;
use zvariant::Value;

type VariantMap<'a> = HashMap<&'a str, &'a Value<'a>>;

#[zbus::dbus_proxy]
trait Notifications {
    async fn notify(
        &self,
        app_name: &str,
        replaces_id: u32,
        app_icon: &str,
        summary: &str,
        body: &str,
        actions: &[&str],
        hints: VariantMap<'_>,
        timeout: i32,
    ) -> zbus::Result<u32>;
}

#[derive(argh::FromArgs)]
/// Simple pacman update notifier
struct Args {
    /// the update check interval in seconds
    #[argh(option, short = 'i')]
    interval: u64,
}

#[tokio::main]
async fn main() -> Result<()> {
    let args: Args = argh::from_env();
    let interval = Duration::from_secs(args.interval);

    let (tx, rx) = mpsc::channel(1);

    tokio::spawn(async move {
        update_check(tx, interval)
            .await
            .map_err(|e| eprintln!("{}", e))
    });

    if let Err(err) = notifier(rx).await {
        eprintln!("{}", err)
    }

    Ok(())
}

async fn update_check(tx: Sender<u64>, interval: Duration) -> Result<()> {
    let mut interval = tokio::time::interval(interval);

    loop {
        interval.tick().await;

        let cmd = Command::new("pacman").arg("-Qqu").output();
        let output = cmd.await?;
        let stdout = String::from_utf8(output.stdout)?;

        let pkg_count = stdout.split('\n').count() as u64;

        tx.send(pkg_count).await?;
    }
}

async fn notifier(mut rx: Receiver<u64>) -> Result<()> {
    let conn = Connection::session().await?;
    let proxy = NotificationsProxy::new(&conn).await?;

    let mut last_check = 0;

    while let Some(pkg_count) = rx.recv().await {
        if pkg_count == 0 || pkg_count == last_check {
            continue;
        }

        last_check = pkg_count;
        let message = format!("{} updates available", pkg_count);

        proxy
            .notify(
                "pacman",
                0,
                "package-x-generic",
                "Pacman",
                message.as_ref(),
                &[],
                VariantMap::new(),
                3000,
            )
            .await?;
    }

    Ok(())
}
